  
#ifndef _calibrater_cmpt_h_
#define _calibrater_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/calibrater_forward.h>

using namespace std;

namespace casac {
           


   class  calibrater  {
       public :


         ~calibrater();
         calibrater();

         bool open( const string& filename,  bool  compress=false,  bool  addcorr=true,  bool  addmodel=true);

         bool selectvis( const variant & time = variant(),  const variant & spw = variant(),  const variant & scan = variant(),  const variant & field = variant(),  const variant & intent = variant(),  const variant & observation = variant(),  const variant & baseline = variant(),  const variant & uvrange = variant(),  const string& chanmode=string("none"),  int  nchan=1,  int  start=0,  int  step=1,  const Quantity& mstart=Quantity(std::vector<double>(1, 0.0),"km/s"),  const Quantity& mstep=Quantity(std::vector<double>(1, 0.0),"km/s"),  const string& msselect=string(""));

         bool setmodel( const string& modelimage);

         bool setptmodel( const std::vector<double>& stokes=initialize_vector(4,(double)0.0, (double)0.0, (double)0.0, (double)0.0));

         bool setapply( const string& type=string(""), 
			  double t=0.0,  const string& table=string(""),  const variant & field = variant(),  const string& interp=string("linear"),  const string& select=string(""),  bool  calwt=false,  const  std::vector<int>& spwmap=initialize_vector(1, (int)-1),  const std::vector<double>& opacity=initialize_vector(1, (double)0.0));

         bool setcallib( const record& callib=initialize_record("") );

         bool validatecallib( const record& callib=initialize_record("") );

         bool setsolve( const string& type=string("MF"),  const variant & t = variant(),  const string& table=string(""),  bool  append=false, 
			  double preavg=-1.0,  bool  phaseonly=false,  const string& apmode=string("AP"),  const variant & refant = variant(),  const string& refantmode=string("flex"),  int  minblperant=4,  bool  solnorm=false,  const string& normtype=string("mean"),  float  minsnr=0.0,  const string& combine=string(""),  int  fillgaps=0,  const string& cfcache=string(""),  float  painc=360.0,  int  fitorder=0,  float  fraction=0.1,  int  numedge=-1,  const string& radius=string(""),  bool  smooth=true,  bool  zerorates=false);

         bool setsolvegainspline( const string& table=string(""),  bool  append=false,  const string& mode=string("PHAS"), 
			  double splinetime=10800, 
			  double preavg=0.0,  int  npointaver=10, 
			  double phasewrap=250,  const variant & refant = variant());

         bool setsolvebandpoly( const string& table=string(""),  bool  append=false,  const variant & t = variant(),  const string& combine=string(""),  int  degamp=3,  int  degphase=3,  bool  visnorm=false,  bool  solnorm=true,  int  maskcenter=0, 
			  double maskedge=5.0,  const variant & refant = variant());

         bool state();

         bool reset( bool  apply=true,  bool  solve=true);

         bool initcalset( int  calset=0);

         bool delmod( bool  otf=false,  const variant & field = variant(),  const variant & spw = variant(),  bool  scr=false);

         bool solve();

         bool correct( const string& applymode=string(""));

         bool corrupt();

         bool initweights( const string& wtmode=string("nyq"),  bool  dowtsp=false,  const string& tsystable=string(""),  const string& gainfield=string(""),  const string& interp=string(""),  const  std::vector<int>& spwmap=std::vector<int>() );

         record* fluxscale( const string& tablein,  const variant & reference = variant(),  const string& tableout=string(""),  const variant & transfer = variant(),  const string& listfile=string(""),  bool  append=false,  const  std::vector<int>& refspwmap=initialize_vector(1, (int)-1),  float  gainthreshold=-1.0,  const string& antenna=string(""),  const string& timerange=string(""),  const string& scan=string(""),  bool  incremental=false,  int  fitorder=1,  bool  display=false);

         bool accumulate( const string& tablein=string(""),  const string& incrtable=string(""),  const string& tableout=string(""),  const variant & field = variant(),  const variant & calfield = variant(),  const string& interp=string("linear"), 
			  double t=-1.0,  const  std::vector<int>& spwmap=initialize_vector(1, (int)-1));

         record* activityrec();

         bool specifycal( const string& caltable=string(""),  const string& time=string(""),  const string& spw=string(""),  const string& antenna=string(""),  const string& pol=string(""),  const string& caltype=string(""),  const std::vector<double>& parameter=initialize_vector(1, (double)1.0),  const string& infile=string(""),  bool  uniform=true);

         bool smooth( const string& tablein,  const string& tableout,  const variant & field = variant(),  const string& smoothtype=string("median"), 
			  double smoothtime=60.0);

         bool rerefant( const string& tablein,  const string& tableout,  const string& refantmode=string("flexible"),  const variant & refant = variant());

         bool listcal( const string& caltable,  const variant & field = variant(),  const variant & antenna = variant(),  const variant & spw = variant(),  const string& listfile=string(""),  int  pagerows=50);

         bool posangcal( const std::vector<double>& posangcor, const  string& tablein, const  string& tableout=string(""));

         bool linpolcor(const  string& tablein=string(""), const  string& tableout=string(""),  const std::vector<std::string>& fields=std::vector<string>(1, ""));

         bool plotcal( const  std::vector<int>& antennas,  const  std::vector<int>& fields,  const  std::vector<int>& spwids,  const string& plottype=string("AMP"),  const string& tablename=string(""),  int  polarization=1,  bool  multiplot=false,  int  nx=1,  int  ny=1,  const string& psfile=string(""));

         std::vector<double>modelfit( const std::vector<bool>&  vary,  int  niter=0, const  string& compshape=string("P"),  const std::vector<double>& par=initialize_vector(3,(double)1.0, (double)0.0, (double)0.0), const  string& file=string(""));

         bool createcaltable( const string& caltable,  const string& partype,  const string& caltype,  bool  singlechan);

         bool updatecaltable( const string& caltable);

         bool close();

         bool done();

         record* parsecallibfile( const string& filein);

         bool setvi( bool  old=false,  bool  quiet=false);


      private :
#include <tools/synthesis/calibrater_private.h>
   }; 

 
}
#endif
