  
#ifndef _imager_cmpt_h_
#define _imager_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/imager_forward.h>

using namespace std;

namespace casac {
           


   class  imager  {
       public :

static int _pixels_advise;
		
static record _cell_advise;
		
static int _facets_advise;
		
static string _phasecenter_advise;
		
static record _bmaj_fitpsf;
		
static record _bmin_fitpsf;
		
static record _bpa_fitpsf;
		
static record _pointsource_sensitivity;
		
static double _relative_sensitivity;
		
static double _sumweights_sensitivity;
		
static record _senrec_sensitivity;
		
static double _pointsource_apparentsens;
		
static double _relative_apparentsens;
		

         ~imager();
         imager();

         bool advise( bool  takeadvice=true, 
			  double amplitudeloss=0.05,  const variant & fieldofview = variant(),  int & _pixels=imager::_pixels_advise,  record& _cell=imager::_cell_advise,  int & _facets=imager::_facets_advise, string& _phasecenter=imager::_phasecenter_advise);

         record* advisechansel(
			  double freqstart=1.0e6, 
			  double freqend=1.1e6, 
			  double freqstep=100.0,  const string& freqframe=string("LSRK"),  const string& msname=string(""),  int  fieldid=0,  bool  getfreqrange=false, const  string& spwselection=string(""));

         bool approximatepsf(const  string& psf=string(""),  bool  async=false);

         bool boxmask(const  string& mask=string(""),  const std::vector<int>& blc=initialize_vector(4,(int)0, (int)0, (int)0, (int)0),  const std::vector<int>& trc=std::vector<int>() , 
			  double value=1.0);

         bool calcuvw( const  std::vector<int>& fields=initialize_vector(1, (int)-1),  const string& refcode=string(""),  bool  reuse=true);

         record* clean(const  string& algorithm=string("clark"),  int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(),  bool  displayprogress=false,  const std::vector<std::string>& model=std::vector<string>(1, ""),  const std::vector<bool>&  keepfixed=initialize_vector(1, (bool)false), const  string& complist=string(""),  const std::vector<std::string>& mask=std::vector<string>(1, ""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  const std::vector<std::string>& residual=std::vector<string>(1, ""),  const std::vector<std::string>& psfimage=std::vector<string>(1, ""),  bool  interactive=false,  int  npercycle=100, const  string& masktemplate=string(""),  bool  async=false);

         bool clipimage(const  string& image=string(""),  const variant & threshold= variant());

         bool clipvis( const variant & threshold= variant());

         bool close();

         bool defineimage( int  nx=128,  int  ny=-1,  const variant & cellx= variant(),  const variant & celly= variant(), const  string& stokes=string("I"),  const variant & phasecenter= variant(), const  string& mode=string("mfs"),  int  nchan=-1,  const variant & start= variant(),  const variant & step= variant(),  const std::vector<int>& spw=initialize_vector(1, (int)0),  const variant & restfreq= variant(), const  string& outframe=string("LSRK"), const  string& veltype=string("radio"),  int  facets=1,  const variant & movingsource= variant(),  const variant & distance= variant(), const  string& projection=string("SIN"));

         bool done();

         record* drawmask(const  string& image=string(""), const  string& mask=string(""),  int  niter=0,  int  npercycle=0, const  string& threshold=string("0 mJy"));

         bool exprmask(const  string& mask=string(""), 
			  double expr=1.0);

         bool feather(const  string& image=string(""), const  string& highres=string(""), const  string& lowres=string(""), const  string& lowpsf=string(""), 
			  double effdishdiam=-1.0,  bool  lowpassfiltersd=false,  bool  async=false);

         bool filter(const  string& type=string("gaussian"),  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  async=false);

         bool fitpsf(const  string& psf,  bool  async=false,  record& _bmaj=imager::_bmaj_fitpsf,  record& _bmin=imager::_bmin_fitpsf,  record& _bpa=imager::_bpa_fitpsf);

         bool fixvis( const  std::vector<int>& fields=initialize_vector(1, (int)-1),  const  std::vector<std::string>& phasedirs=std::vector<string>(1, ""),  const string& refcode=string(""),  const std::vector<double>& distances=initialize_vector(1, (double)0.0),  const string& datacolumn=string("all"));

         bool ft( const std::vector<std::string>& model=std::vector<string>(1, ""), const  string& complist=string(""),  bool  incremental=false, 
			  double phasecentertime=-1.0);

         variant* getweightgrid(const  string& type=string("imaging"),  const std::vector<std::string>& wgtimages=std::vector<string>(1, ""));

         bool linearmosaic( const std::vector<std::string>& images=std::vector<string>(1, ""), const  string& mosaic=string(""), const  string& fluxscale=string(""), const  string& sensitivity=string(""),  const std::vector<int>& fieldids=initialize_vector(1, (int)0),  bool  usedefaultvp=true, const  string& vptable=string(""),  bool  async=false);

         bool make(const  string& image=string(""),  bool  async=false);

         string predictcomp(const  string& objname=string(""), const  string& standard=string(""),  const variant & epoch= variant(),  const std::vector<double>& freqs=initialize_vector(1,(double)1.0e11), const  string& pfx=string("predictcomp"));

         bool makeimage(const  string& type=string("observed"), const  string& image=string(""), const  string& compleximage=string(""),  bool  verbose=true,  bool  async=false);

         bool makemodelfromsd(const  string& sdimage=string(""), const  string& modelimage=string(""), const  string& sdpsf=string(""), const  string& maskimage=string(""));

         bool mask(const  string& image=string(""), const  string& mask=string(""),  const variant & threshold= variant(),  bool  async=false);

         bool mem(const  string& algorithm=string("entropy"),  int  niter=20,  const variant & sigma= variant(),  const variant & targetflux= variant(),  bool  constrainflux=false,  bool  displayprogress=false,  const std::vector<std::string>& model=std::vector<string>(1, ""),  const std::vector<bool>&  keepfixed=initialize_vector(1, (bool)false), const  string& complist=string(""),  const std::vector<std::string>& prior=std::vector<string>(1, ""),  const std::vector<std::string>& mask=std::vector<string>(1, ""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  const std::vector<std::string>& residual=std::vector<string>(1, ""),  bool  async=false);

         bool nnls( const std::vector<std::string>& model=std::vector<string>(1, ""),  const std::vector<bool>&  keepfixed=initialize_vector(1, (bool)false), const  string& complist=string(""),  int  niter=0, 
			  double tolerance=1e-06,  const std::vector<std::string>& fluxmask=std::vector<string>(1, ""),  const std::vector<std::string>& datamask=std::vector<string>(1, ""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  const std::vector<std::string>& residual=std::vector<string>(1, ""),  bool  async=false);

         bool open(const  string& thems=string(""),  bool  compress=false,  bool  usescratch=false);

         bool pb(const  string& inimage=string(""), const  string& outimage=string(""), const  string& incomps=string(""), const  string& outcomps=string(""), const  string& operation=string("apply"),  const variant & pointingcenter= variant(),  const variant & parangle= variant(), const  string& pborvp=string("pb"),  bool  async=false);

         bool plotsummary();

         bool plotuv( bool  rotate=false);

         bool plotvis(const  string& type=string("all"),  int  increment=1);

         bool plotweights( bool  gridded=false,  int  increment=1);

         bool regionmask(const  string& mask=string(""),  const record& region=initialize_record("") ,  const variant & boxes= variant(),  const variant & circles= variant(), 
			  double value=1.0);

         bool regiontoimagemask(const  string& mask=string(""),  const record& region=initialize_record("") ,  const variant & boxes= variant(),  const variant & circles= variant(), 
			  double value=1.0);

         bool residual( const std::vector<std::string>& model=std::vector<string>(1, ""), const  string& complist=string(""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  bool  async=false);

         bool restore( const std::vector<std::string>& model=std::vector<string>(1, ""), const  string& complist=string(""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  const std::vector<std::string>& residual=std::vector<string>(1, ""),  bool  async=false);

         bool updateresidual( const std::vector<std::string>& model=std::vector<string>(1, ""), const  string& complist=string(""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  const std::vector<std::string>& residual=std::vector<string>(1, ""));

         bool sensitivity( bool  async=false,  record& _pointsource=imager::_pointsource_sensitivity, 
			  double& _relative=imager::_relative_sensitivity, 
			  double& _sumweights=imager::_sumweights_sensitivity,  record& _senrec=imager::_senrec_sensitivity);

         bool apparentsens( bool  async=false, 
			  double& _pointsource=imager::_pointsource_apparentsens, 
			  double& _relative=imager::_relative_apparentsens);

         bool setbeam( const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  async=false);

         bool selectvis(const  string& vis=string(""),  const std::vector<int>& nchan=initialize_vector(1, (int)-1),  const std::vector<int>& start=initialize_vector(1, (int)0),  const std::vector<int>& step=initialize_vector(1, (int)1),  const variant & spw= variant(),  const variant & field= variant(),  const variant & baseline= variant(),  const variant & time= variant(),  const variant & scan= variant(), const  string& intent=string(""),  const variant & observation= variant(),  const variant & uvrange= variant(), const  string& taql=string(""),  bool  usescratch=false,  bool  datainmemory=false,  bool  writeaccess=true);

         record* setjy( const variant & field= variant(),  const variant & spw= variant(), const  string& modimage=string(""),  const std::vector<double>& fluxdensity=initialize_vector(4,(double)0.0, (double)0.0, (double)0.0, (double)0.0), const  string& standard=string("SOURCE"),  bool  scalebychan=false,  const std::vector<double>& spix=initialize_vector(1, (double)0.0),  const variant & reffreq= variant(),  const std::vector<double>& polindex=initialize_vector(1, (double)0.0),  const std::vector<double>& polangle=initialize_vector(1, (double)0.0), 
			  double rotmeas=0.0, const  string& time=string(""), const  string& scan=string(""), const  string& intent=string(""), const  string& observation=string(""), const  string& interpolation=string("nearest"));

         bool ssoflux();

         bool setmfcontrol(
			  double cyclefactor=1.5, 
			  double cyclespeedup=-1, 
			  double cyclemaxpsffraction=0.8,  int  stoplargenegatives=2,  int  stoppointmode=-1, 
			  double minpb=0.1, const  string& scaletype=string("NONE"), 
			  double constpb=0.4,  const std::vector<std::string>& fluxscale=std::vector<string>(1, ""),  bool  flatnoise=true);

         bool setoptions(const  string& ftmachine=string("ft"),  int  cache=-1,  int  tile=16, const  string& gridfunction=string("SF"),  const variant & location= variant(), 
			  double padding=1.0, const  string& freqinterp=string("nearest"),  int  wprojplanes=-1, const  string& epjtablename=string(""),  bool  applypointingoffsets=false,  bool  dopbgriddingcorrections=true, const  string& cfcachedirname=string(""), 
			  double rotpastep=5.0, 
			  double pastep=360.0, 
			  double pblimit=0.05,  int  imagetilevol=0,  bool  singleprecisiononly=false,  int  numthreads=-1,  bool  psterm=true,  bool  aterm=true,  bool  mterm=true,  bool  wbawp=false,  bool  conjbeams=true);

         bool setscales(const  string& scalemethod=string("nscales"),  int  nscales=5,  const std::vector<double>& uservector=initialize_vector(3,(double)0.0, (double)3.0, (double)10.0));

         bool setsmallscalebias( float  inbias=0.6);

         bool settaylorterms( int  ntaylorterms=2, 
			  double reffreq=0.0);

         bool setsdoptions(
			  double scale=1.0, 
			  double weight=1.0,  int  convsupport=-1, const  string& pointingcolumntouse=string("DIRECTION"),  const variant & truncate= variant(),  const variant & gwidth= variant(),  const variant & jwidth= variant(), 
			  double minweight=0.,  bool  clipminmax=false);

         bool setvp( bool  dovp=false,  bool  usedefaultvp=true, const  string& vptable=string(""),  bool  dosquint=false,  const variant & parangleinc= variant(),  const variant & skyposthreshold= variant(), const  string& telescope=string(""),  bool  verbose=true);

         bool setweightgrid( const variant & weight = variant(), const  string& type=string("imaging"));

         bool smooth( const std::vector<std::string>& model=std::vector<string>(1, ""),  const std::vector<std::string>& image=std::vector<string>(1, ""),  bool  usefit=true,  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  normalize=true,  bool  async=false);

         bool stop();

         bool summary();

         bool uvrange(
			  double uvmin=0.0, 
			  double uvmax=0.0);

         bool weight(const  string& type=string("natural"), const  string& rmode=string("none"),  const variant & noise= variant(), 
			  double robust=0.0,  const variant & fieldofview= variant(),  int  npixels=0,  bool  mosaic=false,  bool  async=false);

         record* mapextent(const  string& ref=string("J2000"), const  string& movingsource=string(""), const  string& pointingcolumntouse=string("DIRECTION"));

         record* pointingsampling(const  string& pattern=string("raster"), const  string& ref=string("J2000"), const  string& movingsource=string(""), const  string& pointingcolumntouse=string("DIRECTION"), const  string& antenna=string(""));


      private :
#include <tools/synthesis/imager_private.h>
   }; 

 
}
#endif
