//# State.h: global display state
//# Copyright (C) 2009
//# Associated Universities, Inc. Washington DC, USA.
//#
//# This library is free software; you can redistribute it and/or modify it
//# under the terms of the GNU Library General Public License as published by
//# the Free Software Foundation; either version 2 of the License, or (at your
//# option) any later version.
//#
//# This library is distributed in the hope that it will be useful, but WITHOUT
//# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
//# License for more details.
//#
//# You should have received a copy of the GNU Library General Public License
//# along with this library; if not, write to the Free Software Foundation,
//# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
//#
//# Correspondence concerning AIPS++ should be addressed as follows:
//#        Internet email: aips2-request@nrao.edu.
//#        Postal address: AIPS++ Project Office
//#                        National Radio Astronomy Observatory
//#                        520 Edgemont Road
//#                        Charlottesville, VA 22903-2475 USA
//#

#ifndef CASA_DISPLAY_STATE_H_
#define CASA_DISPLAY_STATE_H_

namespace casa { //# NAMESPACE CASA - BEGIN

	namespace display { //# NAMESPACE DISPLAY - BEGIN
		class state {
			friend class stateCleanup;
		public:

			static state &instance( ) {
				return initialized == false ? startup( ) : *singleton;
			}

			// allows classes (primarily in the redisplay chain) to tell if we
			// are currently outputing to a file...
			bool fileOutputMode( );
			// begin file output
			void beginFileOutputMode( );
			// end file output
			void endFileOutputMode( );

		private:

			static state *singleton;

			state( ) : file_output_mode_count_(0) { }
			~state( ) { }

			static state &startup( );
			static void shutdown( );

			int file_output_mode_count_;
			static bool initialized;

		};

		static class stateCleanup {
		public:
			stateCleanup( ) {
				creation_count += 1;
			}
			~stateCleanup( ) {
				if ( --creation_count == 0 ) {
					state::shutdown( );
				}
			}
		private:
			static unsigned int creation_count;
		} local_state_cleanup_object;
	}

}

#endif
